@@ -61,17 +61,19 @@ def load_configuration():
     CONFIG.set('server', 'interfaces'    , getenv('QGSRV_SERVER_INTERFACES', '0.0.0.0'))
     CONFIG.set('server', 'workers'       , getenv('QGSRV_SERVER_WORKERS', '2'))
     CONFIG.set('server', 'timeout'       , getenv('QGSRV_SERVER_TIMEOUT', '20'))
-    CONFIG.set('server', 'enable_filters', getenv('QGSRV_SERVER_ENABLE_FILTERS', 'yes'))
+    CONFIG.set('server', 'enable_filters', getenv('QGSRV_SERVER_ENABLE_FILTERS', 'no'))
     CONFIG.set('server', 'http_proxy'    , getenv('QGSRV_SERVER_HTTP_PROXY', 'no'))
-    CONFIG.set('server', 'proxy_url'     , getenv('QGSRV_SERVER_PROXY_URL' , ''))
-    CONFIG.set('server', 'restartmon'    , getenv('QGSRV_SERVER_RESTARTMON' , ''))
-    CONFIG.set('server', 'pluginpath'    , getenv('QGSRV_SERVER_PLUGINPATH' , ''))
-    CONFIG.set('server', 'ssl'           , getenv('QGSRV_SERVER_SSL' , 'no'))
+    CONFIG.set('server', 'proxy_url'     , getenv('QGSRV_SERVER_PROXY_URL', ''))
+    CONFIG.set('server', 'restartmon'    , getenv('QGSRV_SERVER_RESTARTMON', ''))
+    CONFIG.set('server', 'pluginpath'    , getenv('QGSRV_SERVER_PLUGINPATH', ''))
+    CONFIG.set('server', 'ssl'           , getenv('QGSRV_SERVER_SSL', 'no'))
     CONFIG.set('server', 'ssl_cert'      , getenv('QGSRV_SERVER_SSL_CERT', ''))
-    CONFIG.set('server', 'ssl_key'       , getenv('QGSRV_SERVER_SSL_KEY' , ''))
-    CONFIG.set('server', 'cross_origin'  , getenv('QGSRV_SERVER_CROSS_ORIGIN' , 'yes'))
-    CONFIG.set('server', 'status_page'   , getenv('QGSRV_SERVER_STATUS_PAGE'  , 'no'))
-    CONFIG.set('server', 'allow_headers' , getenv('QGSRV_SERVER_ALLOW_HEADERS' , 'X-Qgis-,X-Lizmap-'))
+    CONFIG.set('server', 'ssl_key'       , getenv('QGSRV_SERVER_SSL_KEY', ''))
+    CONFIG.set('server', 'cross_origin'  , getenv('QGSRV_SERVER_CROSS_ORIGIN', 'yes'))
+    CONFIG.set('server', 'status_page'   , getenv('QGSRV_SERVER_STATUS_PAGE', 'no'))
+    CONFIG.set('server', 'allow_headers' , getenv('QGSRV_SERVER_ALLOW_HEADERS', 'X-Qgis-,X-Lizmap-'))
+    CONFIG.set('server', 'memory_high_water_mark', 
+               getenv('QGSRV_SERVER_MEMORY_HIGH_WATER_MARK', '0.9'))
 
     #
     # Logging
@@ -118,7 +120,9 @@ def load_configuration():
     CONFIG.set('projects.cache', 'force_etag'          , getenv('QGSRV_CACHE_FORCE_ETAG','no')) 
     CONFIG.set('projects.cache', 'default_handler'     , getenv('QGSRV_CACHE_DEFAULT_HANDLER', 'file'))
     CONFIG.set('projects.cache', 'allow_storage_schemes', getenv('QGSRV_CACHE_ALLOW_STORAGE_SCHEMES', '*'))
-    CONFIG.set('projects.cache', 'check_interval'       , getenv('QGSRV_CACHE_CHECK_INTERVAL', '0'))
+    CONFIG.set('projects.cache', 'check_interval'      , getenv('QGSRV_CACHE_CHECK_INTERVAL', '0'))
+    CONFIG.set('projects.cache', 'observers'           , getenv('QGSRV_CACHE_OBSERVERS', ''))
+    CONFIG.set('projects.cache', 'advanced_report'     , getenv('QGSRV_CACHE_ADVANCED_REPORT', 'no'))
 
     # 
     CONFIG.add_section('projects.schemes')
@@ -127,14 +131,18 @@ def load_configuration():
     CONFIG.add_section('zmq')
     # Identity prefix used in 0MQ worker socket 
     CONFIG.set('zmq', 'identity'     , getenv('QGSRV_ZMQ_IDENTITY' ,'OWS-SERVER'))
-    # Address to bind 0MQ socket - used only with proxy/worker configuration
-    CONFIG.set('zmq', 'bindaddr'     , getenv('QGSRV_ZMQ_INADDR'   ,'tcp://*:18080'))
     # Control the maximum lenghth of the waiting queue
     CONFIG.set('zmq', 'maxqueue'     , getenv('QGSRV_ZMQ_MAXQUEUE' ,'1000'))
     # Control the lifetime of requests on the waiting queue 
     CONFIG.set('zmq', 'timeout'      , getenv('QGSRV_ZMQ_TIMEOUT'  ,'15000'))
+    # Host Address to bind/connect 0MQ sockets - used only with proxy/worker configuration
+    CONFIG.set('zmq', 'hostaddr'     , getenv('QGSRV_ZMQ_HOSTADDR'   ,'*'))
+    # Address to bind 0MQ socket - used only with proxy/worker configuration
+    CONFIG.set('zmq', 'bindaddr'     , getenv('QGSRV_ZMQ_INADDR'   ,'tcp://${zmq:hostaddr}:18080'))
     # Address to bind broadcast address to - used only with proxy/worker configuration
-    CONFIG.set('zmq', 'broadcastaddr', getenv('QGSRV_ZMQ_BROADCASTADDR','tcp://*:18090'))
+    CONFIG.set('zmq', 'broadcastaddr', getenv('QGSRV_ZMQ_BROADCASTADDR','tcp://${zmq:hostaddr}:18090'))
+    # Address to bind cache observer - used only with proxy/worker configuration
+    CONFIG.set('zmq', 'cache_observer_addr', getenv('QGSRV_ZMQ_CACHEOBSERVERADDR','tcp://${zmq:hostaddr}:18091'))
 
     #
     # Monitoring (AMQP)
@@ -244,6 +252,7 @@ class ConfigService:
             # Look in environment
             # Note that the section must exists
             if self.allow_env:
+                LOGGER.debug("Looking for option '%s.%s' in environment", section, option)
                 varname  = 'QGSRV_%s_%s' % (section.upper(), option.upper())
                 varname  = functools.reduce( lambda s,c: s.replace(c,'_'), ENV_REPLACE_CHARS, varname)
                 varvalue = os.getenv(varname)
