@@ -36,8 +36,6 @@ class BaseHandler(tornado.web.RequestHandler):
 
     def prepare(self) -> None:
         self.has_body_arguments = len(self.request.body_arguments)>0
-        # Replace query arguments to upper case:
-        self.request.arguments = { k.upper():v for (k,v) in self.request.arguments.items() }
 
     def encode_arguments(self) -> str:
         return '?'+urlencode({k:v[0] for k,v in self.request.arguments.items()})
@@ -49,6 +47,9 @@ class BaseHandler(tornado.web.RequestHandler):
     def set_default_headers(self) -> None:
         """ Override defaults HTTP headers 
         """
+        # XXX By default tornado set Content-Type to xml/text
+        # this may have unwanted side effects
+        self.clear_header("Content-Type")
         self.set_header("Server",f"Py-Qgis-Server {__version__}")
 
     def on_connection_close(self) -> None:
@@ -101,10 +102,8 @@ class BaseHandler(tornado.web.RequestHandler):
         """
         message = self._reason
 
-        if "exc_info" in kwargs:
-            exception = kwargs['exc_info'][1]
-            # Error was caused by a exception
-            message = "{}".format(exception)
+        #if "exc_info" in kwargs:
+        #    exception = kwargs['exc_info'][1]
                
         self.logger.error("%s", message)
         response = dict(status="error" if status_code != 200 else "ok",
@@ -114,21 +113,24 @@ class BaseHandler(tornado.web.RequestHandler):
         self.write_json(response)
         self.finish()
 
-    def proxy_url(self, http_proxy: bool, endpoint: Optional[str]=None) -> str:
+    def proxy_url(self, http_proxy: bool) -> str:
         """ Return the proxy_url
         """
         # Replace the status url with the proxy_url if any
         req = self.request
         if http_proxy:
-            proxy_url = self._cfg.get('proxy_url') or \
-                req.headers.get('X-Forwarded-Url') or \
-                f"{req.protocol}://{req.host}/"
-            if endpoint:
-                proxy_url = f"{proxy_url.rstrip('/')}{endpoint}"
-        else:
-            # No proxy to handle: return the full path
-            proxy_url = f"{req.protocol}://{req.host}{req.path}"
-        return proxy_url
+            # Replacement by static url, use it as base path
+            proxy_url = self._cfg.get('proxy_url')
+            if proxy_url:
+                return f"{proxy_url.rstrip('/')}{req.path}"
+        
+            # Dynamic proxy url
+            proxy_url = req.headers.get('X-Forwarded-Url')
+            if proxy_url:
+                return f"{proxy_url}"
+        
+        # No proxy to handle: return the full path
+        return f"{req.protocol}://{req.host}{req.path}"
 
 
 
@@ -139,4 +141,13 @@ class NotFoundHandler(BaseHandler):
             reason="Invalid resource path."
         )
 
+class ErrorHandler(BaseHandler):
+    def initialize(self, status_code: int, reason: Optional[str]=None) -> None:
+        super().initialize()
+        self.set_status(status_code)
+        self.reason = reason
+
+    def prepare(self) -> None:
+        raise HTTPError(self._status_code, reason=self.reason)
+
 
