@@ -17,11 +17,17 @@ from ..monitor import Monitor
 
 from .basehandler import BaseHandler
 
-from typing import Optional, Awaitable, List, Dict
+from typing import Any, Union, Optional, Awaitable, List, Dict
 
 LOGGER = logging.getLogger('SRVLOG')
 
 
+def _decode( b: Union[str,bytes] ) -> str:
+    if not isinstance(b,str):
+        return b.decode('utf-8')
+    return b
+
+
 class AsyncClientHandler(BaseHandler):
 
     """ Proxy to Qgis 0MQ worker
@@ -64,14 +70,17 @@ class AsyncClientHandler(BaseHandler):
         # see https://github.com/qgis/QGIS/pull/41333
         copy_headers(self._allowed_hdrs)
 
-    async def handle_request(self, method: str, endpoint: Optional[str]=None) -> Awaitable[None]:
+    async def handle_request(self, method: str) -> Awaitable[None]:
         reqtime = time()
+
         try:
+            meta = None
+            response = None
             delta = None
             query = self.encode_arguments()
 
             headers = {}
-            proxy_url = self.proxy_url(self._proxy, endpoint)
+            proxy_url = self.proxy_url(self._proxy)
             if proxy_url: 
                 headers['X-Forwarded-Url']=proxy_url
 
@@ -95,7 +104,7 @@ class AsyncClientHandler(BaseHandler):
             delta  = time() - reqtime
 
             log_rrequest(proxy_url, status, method, query, delta, hdrs)
-           
+          
             # Send response
             for k,v in hdrs.items():
                 self.set_header(k,v)
@@ -121,6 +130,8 @@ class AsyncClientHandler(BaseHandler):
                     # with (potentially empty) chunk
                     self.write(response.data)
 
+            meta = response.metadata
+
         except RequestTimeoutError:
             status = 504
             delta = time() - reqtime
@@ -138,37 +149,50 @@ class AsyncClientHandler(BaseHandler):
         if status >= 500:
             self._stats.num_errors +=1
 
-        if self._monitor:
-            self._monitor.emit( status, self.request.arguments,  delta, 
-                                meta=self.request.headers)
+        # Send monitoring info
+        self.emit( status, delta, meta or {})
 
-    async def get(self, endpoint: Optional[str]=None) -> Awaitable[None]:
+    def emit(self, status: int, response_time: float, meta: Dict) -> None:
+        if not self._monitor:
+            return
+
+        if meta:
+            LOGGER.debug("### Adv. Metrics => %s", meta)
+
+        params = self.get_monitor_params()
+        if params:
+            params.update(
+                # RESPONSE TIME MUST BE IN MILLISECONDS
+                RESPONSE_TIME = int(response_time*1000.0),
+                RESPONSE_STATUS  = status,
+                RESPONSE_MEMUSED = meta.get('mem_used',0)
+            )
+            self._monitor.emit( params, meta=self.request.headers )
+
+    async def get(self) -> Awaitable[None]:
         """ Handle Get method
         """
-        await self.handle_request('GET', endpoint)
+        await self.handle_request('GET')
 
-    async def post(self, endpoint: Optional[str]=None) -> Awaitable[None]:
+    async def post(self) -> Awaitable[None]:
         """ Handle Post method
         """
-        await self.handle_request('POST', endpoint)
-        
-    def options(self, endpoint: Optional[str]=None) -> None:
+        await self.handle_request('POST')
+
+    async def head(self) -> Awaitable[None]:
+        """ Handle HEAD method
+        """
+        await self.handle_request('HEAD')
+
+    def options(self) -> None:
         """ Implement OPTIONS for validating CORS
         """
         self.set_option_headers('GET, POST, OPTIONS')
 
-    def head(self, endpoint: Optional[str]=None) -> Awaitable[None]:
-        """ Handle HEAD method
-
-            HEAD method is not forwarded because qgis server 
-            return 501.
-            
-            Because of this the actuel HEAD method does not return anything meaningful about 
-            the resources referenced from the url - may this will change it the future.
+    def get_monitor_params( self ) -> Optional[Dict[str,Any]]:
+        """ Emit monitoring info
         """
-        pass
-
-
+        return None
 
 class OwsHandler(AsyncClientHandler):
 
@@ -176,55 +200,69 @@ class OwsHandler(AsyncClientHandler):
         super().initialize(*args, **kwargs )
         self.ogc_scheme = 'OWS'
 
+    MONITOR_ARGUMENTS = (
+        'MAP',
+        'SERVICE',
+        'REQUEST',
+    )
 
-class _FilterHandlerMixIn:
-    """ Handle filter handlers
-    """
-    def initialize(self, filters: Optional[List]=None, **kwargs) -> None:
-        super().initialize(**kwargs)
-        
-        self._filters = filters or []
+    def get_monitor_params( self ) -> Dict[str,Any]:
+        """ Override
+        """
+        args = self.request.arguments
+        params = { k:_decode(args.get(k,["__unknown__"])[0]) for k in self.MONITOR_ARGUMENTS }
+        return params
 
-    async def prepare(self) -> Awaitable[None]:
-        # Handle filters
+    def prepare(self) -> None:
         super().prepare()
-        for filt in self._filters:
-            await filt.apply( self )
-
-
-class OwsFilterHandler(_FilterHandlerMixIn,OwsHandler):
-    pass
+        # Replace query arguments to upper case: (it's ok for OWS)
+        self.request.arguments = { k.upper():v for (k,v) in self.request.arguments.items() }
 
 
 class OwsApiHandler(AsyncClientHandler):
     """ Handle Qgis api
     """
-    def initialize( self, *args, **kwargs ) -> None:
-        super().initialize(*args, **kwargs )
+    def initialize( self, service: str, **kwargs ) -> None:
+        super().initialize(**kwargs )
         self.ogc_scheme = 'OAF'
+        self._service_name = service.upper()
 
-    async def get(self, endpoint: Optional[str]=None) -> Awaitable[None]:
-        """ Fix issue with the landing page api when not 
-            specifying index.html.
+    def prepare(self) -> None:
+        super().prepare()
+        # Replace MAP key with uppercase
+        args = self.request.arguments
+        if 'MAP' in args:
+            return 
+        for k in args:
+            if k.upper() == 'MAP':
+                key = k
+                val = args[k]
+                break
+        else:
+            return
+        del args[key]
+        args['MAP'] = val
+
+    async def delete(self) -> Awaitable[None]:
+        await self.handle_request('DELETE')
+
+    async def put(self) -> Awaitable[None]:
+        await self.handle_request('PUT')
+
+    async def patch(self) -> Awaitable[None]:
+        await self.handle_request('PATCH')
+
+    async def options(self) -> Awaitable[None]:
+        await self.handle_request('OPTIONS')
+
+    def get_monitor_params( self ) -> None:
+        """ Override
         """
-        await super().get(endpoint)
- 
-    async def delete(self, endpoint: Optional[str]=None) -> Awaitable[None]:
-        await self.handle_request('DELETE', endpoint)
-
-    async def head(self, endpoint: Optional[str]=None) -> Awaitable[None]:
-        await self.handle_request('HEAD', endpoint)
-
-    async def put(self, endpoint: Optional[str]=None) -> Awaitable[None]:
-        await self.handle_request('PUT', endpoint)
-
-    async def patch(self, endpoint: Optional[str]=None) -> Awaitable[None]:
-        await self.handle_request('PATCH', endpoint)
-
-    async def options(self, endpoint: Optional[str]=None) -> Awaitable[None]:
-        await self.handle_request('OPTIONS', endpoint)
+        params = dict(
+            MAP = self.request.arguments.get('MAP','__unknown__'),
+            SERVICE = self._service_name,
+            REQUEST = self.request.path,
+        )
+        return params
 
 
-class OwsApiFilterHandler(_FilterHandlerMixIn, OwsApiHandler):
-    pass
-
