@@ -11,18 +11,20 @@
 
 import logging
 import urllib.parse
+import traceback
 
 from urllib.parse import urlparse, urlunparse, urljoin, parse_qs
-from typing import Any, Tuple, Optional, Sequence, NamedTuple
+from typing import Any, Tuple, Optional, Sequence, NamedTuple, Callable, Generator
 from collections import OrderedDict
 from pathlib import Path
 from datetime import datetime
 from enum import Enum
-from itertools import chain
 
 from ..utils.lru import lrucache
 from ..config import confservice
 
+from .types import UpdateState
+
 from qgis.PyQt.QtCore import Qt 
 from qgis.core import (QgsApplication,
                        QgsProjectBadLayerHandler, 
@@ -75,7 +77,6 @@ def _merge_qs( query1: str, query2: str ) -> str:
 class CacheType(Enum): 
     LRU = 'lru'
     STATIC = 'static'
-    ALL = 'all'
 
 
 class QgisStorageHandler:
@@ -85,15 +86,9 @@ class QgisStorageHandler:
     def __init__(self):
         pass
 
-    def get_project( self, url: urllib.parse.ParseResult, strict: Optional[bool]=None,
-                     project: Optional[QgsProject]=None,
-                     timestamp: Optional[datetime]=None) -> Tuple[QgsProject, datetime]:
-        """ Create or return a project
-        """
-        uri = urlunparse(url)
-
+    def get_storage_metadata( self, uri: str ):
         # Check out for storage
-        storage = QgsApplication.projectStorageRegistry().projectStorageFromUri( uri )
+        storage = QgsApplication.projectStorageRegistry().projectStorageFromUri(uri)
         if not storage:
             LOGGER.error("No project storage found for %s", uri)
             raise FileNotFoundError(uri)
@@ -101,11 +96,27 @@ class QgisStorageHandler:
         if not res:
             LOGGER.error("Failed to read storage metadata for %s", uri)
             raise FileNotFoundError(uri)
+        return metadata
+
+    def get_modified_time( self, url: urllib.parse.ParseResult) -> datetime:
+        """ Return the modified date time of the project referenced by its url
+        """
+        metadata = self.get_storage_metadata(urlunparse(url))
+        return metadata.lastModified.toPyDateTime()
+
+    def get_project( self, url: urllib.parse.ParseResult,
+                     project: Optional[QgsProject]=None,
+                     timestamp: Optional[datetime]=None) -> Tuple[QgsProject, datetime]:
+        """ Create or return a project
+        """
+        uri = urlunparse(url)
+
+        metadata = self.get_storage_metadata(uri)
         modified_time = metadata.lastModified.toPyDateTime()
 
         if timestamp is None or timestamp < modified_time:
             cachmngr  = componentmanager.get_service('@3liz.org/cache-manager;1')
-            project   = cachmngr.read_project(uri, strict=strict)
+            project   = cachmngr.read_project(uri)
             timestamp = modified_time
 
         return project, timestamp
@@ -138,6 +149,7 @@ class QgsCacheManager:
         self._disable_owsurls      = cnf.getboolean('disable_owsurls', fallback=False)
         self._aliases = {}
         self._default_scheme = cnf.get('default_handler')
+        self._observers = []   
 
         allowed_schemes = cnf.get('allow_storage_schemes')
         if allowed_schemes != '*':
@@ -147,9 +159,34 @@ class QgsCacheManager:
         # Set the base url for file protocol
         self._aliases['file'] = 'file:///%s/' % cnf.get('rootdir').strip('/')
 
+        if self._disable_getprint:
+            LOGGER.info("** Cache: Getprint disabled")
+
+        if self._trust_layer_metadata:
+            LOGGER.info("** Cache: Trust Layer Metadata on")
+
         # Load protocol handlers
         componentmanager.register_entrypoints('qgssrv_contrib_protocol_handler')
 
+    def add_observer( self, observer: Callable[[str,datetime,int],None] ) -> None: 
+        """ Add observer for cache invalidation
+        """
+        self._observers.append(observer)
+
+    def notify_observers( self, key: str, modified_time: datetime,
+                          state: UpdateState) -> None:
+        """ Notify all observers
+        """
+        if not self._observers:
+            return
+
+        modified_time = modified_time.replace(microsecond=0)
+        for obs in self._observers:
+            try:
+                obs(key, modified_time, int(state))
+            except Exception:
+                LOGGER.critical("Uncaugh error in observer: %s\n%s", obs, traceback.format_exc())
+
     @property
     def trust_mode_on(self) -> bool:
         return self._trust_layer_metadata
@@ -158,28 +195,12 @@ class QgsCacheManager:
     def strict_mode_on(self) -> bool:
         return self._strict_check
 
-    def clear(self) -> None:
-        """ Clear the whole cache
-        """
-        self._lru_cache.clear()
-        self._static_cache.clear()
-
-    def items(self, cachetype: CacheType = CacheType.ALL) -> Sequence[Tuple[str,CacheDetails]]:
-        if cachetype == CacheType.ALL:
-            return chain(self._lru_cache.items(), self._static_cache.items())
-        elif cachetype == CacheType.LRU:
+    def items(self, cachetype: CacheType = CacheType.LRU) -> Sequence[Tuple[str,CacheDetails]]:
+        if cachetype == CacheType.LRU:
             return self._lru_cache.items()
         elif cachetype == CacheType.STATIC:
             return self._static_cache.items()
 
-    def remove_entry(self, key: str) -> None:
-        """ Remove cache entry
-        """
-        if key in self._static_cache:
-            del self._static_cache[key]
-        else:
-            del self._lru_cache[key]
-
     def resolve_alias(self, key: str ) -> urllib.parse.ParseResult:
         """ Resolve scheme from configuration variables
         """
@@ -235,65 +256,125 @@ class QgsCacheManager:
 
         return store 
 
-    def refresh(self):
+    def refresh(self) -> Generator[Tuple[str,UpdateState], None, None]:
         """ Refresh all entries
 
             keys are returned from most recently user to the last recently,
             then we have to update in reverse for preserving order 
         """
-        keys = reversed([k for k,_ in self.items()])
-        for key in keys:
-            self.update_entry(key)
+        # Update static cache;
+        for key,_ in self.items(CacheType.STATIC):
+            self.update_static_entry(key)
 
-    def get_project(self, key: str, strict: Optional[bool]=None, refresh: bool=True) -> Tuple[QgsProject,datetime,bool]:
-        """ Load project 
+        # Update both caches
+        keys = reversed([k for k,_ in self.items(CacheType.LRU)])
+        return ((key, self.update_entry(key)) for key in keys)
+
+    def peek(self, key:str, cachetype: Optional[CacheType] = None) -> CacheDetails:
+        """ Return cache details 
         """
-        # Get details for the project
-        details = self._lru_cache.peek(key) or self._static_cache.get(key)
+        if cachetype == CacheType.LRU:
+            return self._lru_cache.peek(key)
+        elif cachetype == CacheType.STATIC:
+            return self._static_cache.get(key)
+        else:
+            return self._lru_cache.peek(key) or self._static_cache.get(key)
 
-        # We are asked not to refresh the entry
-        # return what is in cache
-        if details and not refresh:
-            return details.project, details.timestamp, False
+    def get_modified_time(self, key: str, from_cache: bool=True) -> datetime:
+        """ Get the modified time for the given project uri
+        """
+        details = self.peek(key)
+        if details:
+            if from_cache:
+                # Return from cached resource
+                return details.timestamp.replace(microsecond=0)
 
+            # Trust Qgis to return modified time
+            last_modified = details.project.lastModified()
+            if not last_modified.isValid():
+                # Occurs if resource is not valid 
+                LOGGER.error("QgsProject::lastModified() returned invalid date time for %s", 
+                             details.project.fileName())
+                raise FileNotFoundError(key)
+            last_modified = last_modified.toPyDateTime()
+        else:
+            # Get modified 
+            url = self.resolve_alias(key)
+            store = self.get_protocol_handler(key, url.scheme)
+            last_modified = store.get_modified_time(url)
+
+        return last_modified.replace(microsecond=0)
+
+    def _get_project_details(self, key: str, 
+                             details: Optional[CacheDetails]) -> Tuple[CacheDetails, bool]: 
+        """ Return updated project details
+        """
         url   = self.resolve_alias(key)
         store = self.get_protocol_handler(key, url.scheme)
 
         if details is not None:
-            project, timestamp  = store.get_project( url, strict=strict, **details._asdict())
-            needupdate = timestamp != details.timestamp
+            project, timestamp  = store.get_project(url, **details._asdict())
+            update = UpdateState.UPDATED if timestamp != details.timestamp else UpdateState.UNCHANGED
         else:
-            project, timestamp = store.get_project(url, strict=strict)
-            needupdate = True
+            project, timestamp = store.get_project(url)
+            update = UpdateState.INSERTED
 
-        return project, timestamp, needupdate
-        
-    def update_entry(self, key: str, static_cache: bool=False, refresh: bool=True) -> bool:
-        """ Update the cache
+        return CacheDetails(project, timestamp), update
 
-            :param key: The key of the entry to update
-            :return: true if the entry has been updated
+    def update_static_entry( self, key: str ) -> UpdateState:
+        """ Update static cache
         """
-        project, timestamp, updated = self.get_project(key, refresh=refresh)
-        if static_cache or key in self._static_cache:
-            self._static_cache[key] = CacheDetails(project, timestamp)
+        details, update = self._get_project_details(key, self._static_cache.get(key))
+        self._static_cache[key] = details
+        return update
+
+    def update_entry(self, key: str) -> UpdateState:
+        """ Update LRU entry
+        """
+        # Get details for the project
+        details = self._lru_cache.peek(key)
+
+        # Lookup in static cache
+        if not details and key in self._static_cache:
+            details = self._static_cache[key]
+            details, update = self._get_project_details(key, details)
+            if update == UpdateState.UPDATED:
+                # Update the static cache
+                self._static_cache[key] = details
+            # LRU is updated with new content
+            update = UpdateState.INSERTED
         else:
-            self._lru_cache[key] = CacheDetails(project, timestamp)
-        return updated
+            details, update = self._get_project_details(key, details)
 
-    def peek(self, key: str) -> Optional[CacheDetails]:
-        """ Return entry if it exists
-        """
-        return self._lru_cache.peek(key) or self._static_cache.get(key)
+        self._lru_cache[key] = details
 
-    def lookup(self, key: str, refresh: bool=True) -> Tuple[QgsProject, bool]:
+        # Notify
+        if update:
+            LOGGER.info("Updated project '%s' in cache", key),
+            self.notify_observers(key, details.timestamp, update)
+
+        return update
+
+    def lookup(self, key: str, refresh: bool = True) -> Tuple[QgsProject, UpdateState]:
         """ Lookup entry from key
+
+            If refresh is False, return actual cache 
+            content without refreshing/updating the entry.
         """
-        updated = self.update_entry(key, refresh=refresh)
-        if key in self._static_cache:
-            return self._static_cache[key].project, updated
-        else:
-            return self._lru_cache[key].project, updated
+        if not refresh:
+            # Lookup LRU
+            details = self.peek(key, CacheType.LRU)
+            if details:
+                return details.project, UpdateState.UNCHANGED
+            # Update from static cache
+            details = self.peek(key, CacheType.STATIC)
+            if details:
+                self._lru_cache[key] = details
+                return details.project, UpdateState.UPDATED
+        
+        # Not found in cache, update the actual entry
+        update = self.update_entry(key)
+        return self._lru_cache[key].project, update
 
     def prepare_project(self, project: QgsProject ) -> None:
         """ Set project configuration
@@ -307,7 +388,7 @@ class QgsCacheManager:
             project.writeEntry("WCSUrl","/", "")
             project.writeEntry("WMTSUrl","/", "")
 
-    def read_project(self, uri: str, strict: Optional[bool]=None) -> QgsProject:
+    def read_project(self, uri: str) -> QgsProject:
         """ Read project from path
 
             May be used by protocol-handlers to instanciate project
@@ -326,8 +407,7 @@ class QgsCacheManager:
         if not project.read(uri,  readflags):
             raise RuntimeError(f"Failed to read Qgis project {uri}")
 
-        strict = self._strict_check if strict is None else strict
-        if strict and not badlayerh.validateLayers(project):
+        if self._strict_check and not badlayerh.validateLayers(project):
             raise StrictCheckingError
 
         self.prepare_project(project)
@@ -380,7 +460,7 @@ def preload_projects_file( path: Path, cacheservice: QgsCacheManager ) ->  int:
         for p in filter(None,(line.strip('\n ').partition('#')[0] for line in fp.readlines())):
             p = p.strip(' ')
             try:
-                cacheservice.update_entry(p, static_cache=True)
+                cacheservice.update_static_entry(p)
             except StrictCheckingError:
                 LOGGER.error("Preload: '%s' as invalid layers - strict mode on" , p)
             except PathNotAllowedError:
