@@ -23,7 +23,7 @@ from glob import glob
 from multiprocessing import Process
 from multiprocessing.util import Finalize
 
-from typing import Callable, Awaitable, Optional, Dict, List
+from typing import Callable, Awaitable, Dict, List
 
 from .zeromq.supervisor import Supervisor
 from .zeromq.pool import Pool
@@ -34,6 +34,13 @@ from .qgsworker import QgsRequestHandler
 
 from pyqgisservercontrib.core.watchfiles import watchfiles
 
+
+try:
+    import psutil
+except ImportError:
+    psutil = None
+
+
 LOGGER=logging.getLogger('SRVLOG')
 
 class _RestartHandler:
@@ -81,7 +88,8 @@ class _RestartHandler:
 class _Server:
 
     def __init__(self, broadcastaddr: str, pool: Process,  timeout: int,
-                 num_workers: int) -> None:
+                 num_workers: int,
+                 high_water_mark: float = 1.0) -> None:
 
         ctx = zmq.Context.instance()
         pub = ctx.socket(zmq.PUB)
@@ -93,6 +101,8 @@ class _Server:
         self._sock = pub
         self._num_workers = num_workers
 
+        self._high_water_mark = high_water_mark
+
         LOGGER.debug("Started pool server")
         self._pool = pool
         self._supervisor = None
@@ -111,9 +121,14 @@ class _Server:
 
     async def healthcheck(self) -> Awaitable[None]:
         while True:
+            # Check for exitcode
             if self._pool.exitcode is not None and self._pool.exitcode != 0:
                 LOGGER.critical("Pool failure, exiting because of unrecoverable error...")
                 raise SystemExit(1)
+            # Check high water mark
+            if self.memory_fraction() > self._high_water_mark:
+                LOGGER.critical("High memory water mark reached: restarting workers %s", self._high_water_mark)
+                self.restart()
             await asyncio.sleep(5)
     
     def start_supervisor(self):
@@ -142,14 +157,12 @@ class _Server:
     def terminate(self):
         """ Terminate handler
         """
-        LOGGER.info("Stopping pool server")
         self._restart_handler.close()
         if self._healthcheck:
             self._healthcheck.cancel()
         self._sock.close()
         if self._supervisor:
             self._supervisor.stop()
-        LOGGER.info("Stopping worker pool")
         self._terminate()
 
     def broadcast(self, command: bytes ) -> None:
@@ -184,6 +197,16 @@ class _Server:
                 break
         return self._supervisor.reports
 
+    def memory_fraction(self) -> float:
+        """ Return total memory fraction used by pool
+        """
+        if psutil is not None:
+            p = psutil.Process(self._pool.pid)
+            mem = sum(child.memory_percent() for child in p.children(recursive=True))
+        else:
+            mem = 0
+        return mem / 100.0
+
 
 def create_poolserver(numworkers: int) -> _Server:
     """ Run workers pool in its own process
@@ -195,22 +218,17 @@ def create_poolserver(numworkers: int) -> _Server:
     broadcastaddr = confservice['zmq']['broadcastaddr']
     timeout       = confservice['server'].getint('timeout')
 
-    maxcycles = confservice.getint('server', 'maxcycles', fallback=None)
-    if maxcycles is not None:
-        if maxcycles <= 0:
-            maxcycles = None
-        else:
-            LOGGER.info(f"Max cycles limit set to {maxcycles}")
+    high_water_mark = float(confservice['server']['memory_high_water_mark'])
 
-    p = Process(target=run_worker_pool, args=(numworkers, broadcastaddr, router, maxcycles))
+    p = Process(target=run_worker_pool, args=(numworkers, broadcastaddr, router))
     p.start()
 
-    poolserver = _Server(broadcastaddr, p, timeout, numworkers)
+    poolserver = _Server(broadcastaddr, p, timeout, numworkers, 
+                         high_water_mark = high_water_mark)
     return poolserver
 
 
-def run_worker_pool(numworkers: int, broadcastaddr: str, router: str, 
-                    maxcycles: Optional[int]=None) -> None:
+def run_worker_pool(numworkers: int, broadcastaddr: str, router: str) -> None:
     """ Run a qgis worker pool
 
         Ensure that child processes run in the main thread
@@ -223,7 +241,7 @@ def run_worker_pool(numworkers: int, broadcastaddr: str, router: str,
 
     LOGGER.info("Starting worker pool")
     pool = Pool( numworkers, target=QgsRequestHandler.run, args=(router,),
-                 kwargs={ 'broadcastaddr': broadcastaddr, 'maxcycles': maxcycles } )
+                 kwargs={ 'broadcastaddr': broadcastaddr } )
 
     signal.signal(signal.SIGTERM,term_signal)
 
